@@ -27,14 +27,14 @@
 #include <libtelepathy/tp-chan-iface-media-signalling-gen.h>
 #include <libtelepathy/tp-media-stream-handler-gen.h>
 #include <libtelepathy/tp-helpers.h>
+#include <libtelepathy/tp-props-iface.h>
 
 #include "common/telepathy-errors.h"
 
-#include "types.h"
-#include "stream.h"
-#include "session.h"
-
 #include "channel.h"
+#include "session.h"
+#include "stream.h"
+#include "types.h"
 
 G_DEFINE_TYPE (TpStreamEngineChannel, tp_stream_engine_channel, G_TYPE_OBJECT);
 
@@ -48,15 +48,22 @@ struct _TpStreamEngineChannelPrivate
   TpChan *channel_proxy;
   DBusGProxy *streamed_media_proxy;
 
+  TpStreamEngineStreamProperties props;
+
   gulong channel_destroy_handler;
 
-  guint output_volume;
-  gboolean output_mute;
-  gboolean input_mute;
-
   gchar *connection_path;
 };
 
+enum
+{
+  PROP_NAT_TRAVERSAL = 0,
+  PROP_STUN_SERVER,
+  PROP_STUN_PORT,
+  PROP_GTALK_P2P_RELAY_TOKEN,
+  NUM_TP_PROPERTIES
+};
+
 enum
 {
   CLOSED,
@@ -102,6 +109,15 @@ tp_stream_engine_channel_dispose (GObject *object)
       priv->channel_proxy = NULL;
     }
 
+  g_free (priv->props.nat_traversal);
+  priv->props.nat_traversal = NULL;
+
+  g_free (priv->props.stun_server);
+  priv->props.stun_server = NULL;
+
+  g_free (priv->props.relay_token);
+  priv->props.relay_token = NULL;
+
   if (G_OBJECT_CLASS (tp_stream_engine_channel_parent_class)->dispose)
     G_OBJECT_CLASS (tp_stream_engine_channel_parent_class)->dispose (object);
 }
@@ -128,12 +144,9 @@ tp_stream_engine_channel_class_init (TpStreamEngineChannelClass *klass)
 static void
 tp_stream_engine_channel_init (TpStreamEngineChannel *self)
 {
-  TpStreamEngineChannelPrivate *priv = CHANNEL_PRIVATE (self);
+/*  TpStreamEngineChannelPrivate *priv = CHANNEL_PRIVATE (self); */
 
   self->sessions = g_ptr_array_new ();
-
-  /* sensible default */
-  priv->output_volume = (65535 * 7) / 10;
 }
 
 static void
@@ -153,7 +166,7 @@ add_session (TpStreamEngineChannel *self,
   session = g_object_new (TP_STREAM_ENGINE_TYPE_SESSION, NULL);
 
   if (!tp_stream_engine_session_go (session, bus_name, priv->connection_path,
-      session_handler_path, self->channel_path, type))
+      session_handler_path, self->channel_path, type, &(priv->props)))
     {
       g_critical ("couldn't create session");
     }
@@ -240,7 +253,7 @@ get_session_handlers_reply (DBusGProxy *proxy,
                             gpointer user_data)
 {
   TpStreamEngineChannel *self = TP_STREAM_ENGINE_CHANNEL (user_data);
-  int i;
+  guint i;
 
   if (error)
     g_critical ("Error calling GetSessionHandlers: %s", error->message);
@@ -271,6 +284,117 @@ get_session_handlers_reply (DBusGProxy *proxy,
     }
 }
 
+static const gchar *prop_names[NUM_TP_PROPERTIES] = {
+    "nat-traversal",
+    "stun-server",
+    "stun-port",
+    "gtalk-p2p-relay-token"
+};
+
+static void
+update_prop_str (TpPropsIface *iface,
+                 guint prop_id,
+                 gchar **value)
+{
+  GValue tmp = {0, };
+
+  g_free (*value);
+  *value = NULL;
+
+  if (tp_props_iface_property_flags (iface, prop_id) & TP_PROPERTY_FLAG_READ)
+    {
+      g_value_init (&tmp, G_TYPE_STRING);
+
+      if (tp_props_iface_get_value (iface, prop_id, &tmp))
+        {
+          *value = g_value_dup_string (&tmp);
+          g_debug ("got %s = %s", prop_names[prop_id], *value);
+        }
+
+      g_value_unset (&tmp);
+    }
+}
+
+static void
+update_prop_uint (TpPropsIface *iface,
+                  guint prop_id,
+                  guint16 *value)
+{
+  GValue tmp = {0, };
+
+  *value = 0;
+
+  if (tp_props_iface_property_flags (iface, prop_id) & TP_PROPERTY_FLAG_READ)
+    {
+      g_value_init (&tmp, G_TYPE_UINT);
+
+      if (tp_props_iface_get_value (iface, prop_id, &tmp))
+        {
+          *value = g_value_get_uint (&tmp);
+          g_debug ("got %s = %u", prop_names[prop_id], *value);
+        }
+
+      g_value_unset (&tmp);
+    }
+}
+
+static void
+update_prop (TpPropsIface *iface,
+             TpStreamEngineStreamProperties *props,
+             guint prop_id)
+{
+  switch (prop_id)
+    {
+    case PROP_NAT_TRAVERSAL:
+      update_prop_str (iface, PROP_NAT_TRAVERSAL, &(props->nat_traversal));
+      break;
+    case PROP_STUN_SERVER:
+      update_prop_str (iface, PROP_STUN_SERVER, &(props->stun_server));
+      break;
+    case PROP_STUN_PORT:
+      update_prop_uint (iface, PROP_STUN_PORT, &(props->stun_port));
+      break;
+    case PROP_GTALK_P2P_RELAY_TOKEN:
+      update_prop_str (iface, PROP_GTALK_P2P_RELAY_TOKEN,
+          &(props->relay_token));
+      break;
+    default:
+      g_debug ("%s: ignoring unknown property id %u", G_STRFUNC, prop_id);
+    }
+}
+
+static void
+cb_property_changed (TpPropsIface *iface,
+                     guint prop_id,
+                     TpPropsChanged changed,
+                     gpointer user_data)
+{
+  TpStreamEngineChannel *self = TP_STREAM_ENGINE_CHANNEL (user_data);
+  TpStreamEngineChannelPrivate *priv = CHANNEL_PRIVATE (self);
+  TpStreamEngineStreamProperties *props = &(priv->props);
+
+  update_prop (iface, props, prop_id);
+}
+
+static void
+cb_properties_ready (TpPropsIface *iface,
+                     gpointer user_data)
+{
+  TpStreamEngineChannel *self = TP_STREAM_ENGINE_CHANNEL (user_data);
+  TpStreamEngineChannelPrivate *priv = CHANNEL_PRIVATE (self);
+  TpStreamEngineStreamProperties *props = &(priv->props);
+  guint i;
+
+  for (i = 0; i < NUM_TP_PROPERTIES; i++)
+    update_prop (iface, props, i);
+
+  g_signal_handlers_disconnect_by_func (iface,
+      G_CALLBACK (cb_properties_ready), self);
+
+  g_signal_connect (iface, "properties-changed",
+      G_CALLBACK (cb_property_changed), self);
+}
+
 gboolean
 tp_stream_engine_channel_go (
   TpStreamEngineChannel *self,
@@ -282,6 +406,7 @@ tp_stream_engine_channel_go (
   GError **error)
 {
   TpStreamEngineChannelPrivate *priv = CHANNEL_PRIVATE (self);
+  TpPropsIface *props;
   DBusGProxy *media_signalling;
 
   g_assert (NULL == priv->channel_proxy);
@@ -305,16 +430,6 @@ tp_stream_engine_channel_go (
       return FALSE;
     }
 
-  /* TODO: check for group interface
-  chan_interfaces = (GSList *) tp_chan_local_get_interface_objs(priv->chan);
-
-  if (chan_interfaces == NULL)
-  {
-    g_error("Channel does not have interfaces.");
-    exit(1);
-  }
-  */
-
   priv->channel_destroy_handler = g_signal_connect (
     priv->channel_proxy, "destroy", G_CALLBACK (channel_destroyed), self);
 
@@ -322,6 +437,23 @@ tp_stream_engine_channel_go (
                                G_CALLBACK (channel_closed),
                                self, NULL);
 
+  props = (TpPropsIface *) tp_chan_get_interface (priv->channel_proxy,
+      TELEPATHY_PROPS_IFACE_QUARK);
+
+  /* fail gracefully if there's no properties interface */
+  if (props != NULL)
+    {
+      tp_props_iface_set_mapping (props,
+          "nat-traversal", PROP_NAT_TRAVERSAL,
+          "stun-server", PROP_STUN_SERVER,
+          "stun-port", PROP_STUN_PORT,
+          "gtalk-p2p-relay-token", PROP_GTALK_P2P_RELAY_TOKEN,
+          NULL);
+
+      g_signal_connect (props, "properties-ready",
+          G_CALLBACK (cb_properties_ready), self);
+    }
+
   priv->streamed_media_proxy = tp_chan_get_interface (priv->channel_proxy,
       TELEPATHY_CHAN_IFACE_STREAMED_QUARK);
 
