@@ -27,8 +27,6 @@
 #include <libtelepathy/tp-conn.h>
 #include <libtelepathy/tp-helpers.h>
 #include <libtelepathy/tp-interfaces.h>
-#include <libtelepathy/tp-props-iface.h>
-#include <libtelepathy/tp-props-iface.h>
 #include <libtelepathy/tp-media-stream-handler-gen.h>
 
 #include <farsight/farsight-session.h>
@@ -40,17 +38,9 @@
 #include "common/telepathy-errors.h"
 
 #include "tp-stream-engine.h"
-#include "types.h"
-
-#ifdef MAEMO_OSSO_SUPPORT
-#include "media-engine-gen.h"
-
-#define MEDIA_SERVER_SERVICE_NAME "com.nokia.osso_media_server"
-#define MEDIA_SERVER_INTERFACE_NAME "com.nokia.osso_media_server"
-#define MEDIA_SERVER_SERVICE_OBJECT "/com/nokia/osso_media_server"
-#endif
-
 #include "stream.h"
+#include "types.h"
+#include "util.h"
 
 G_DEFINE_TYPE (TpStreamEngineStream, tp_stream_engine_stream, G_TYPE_OBJECT);
 
@@ -72,40 +62,19 @@ struct _TpStreamEngineStreamPrivate
   gchar *channel_path;
 
   DBusGProxy *stream_handler_proxy;
-  TpPropsIface *conn_props;
   TpConn *connection_proxy;
 
   FarsightStream *fs_stream;
   guint state_changed_handler_id;
 
-  gchar *stun_server;
-  guint stun_port;
+  gboolean playing;
+  FarsightStreamState state;
+  FarsightStreamDirection dir;
 
   guint output_volume;
   gboolean output_mute;
   gboolean input_mute;
   guint output_window_id;
-
-  gboolean stream_started;
-  gboolean stream_start_scheduled;
-  gboolean got_connection_properties;
-  gboolean candidate_preparation_required;
-
-#ifdef MAEMO_OSSO_SUPPORT
-  DBusGProxy *media_server_proxy;
-#endif
-};
-
-enum
-{
-  CONN_PROP_STUN_SERVER = 0,
-  CONN_PROP_STUN_PORT,
-  CONN_PROP_STUN_RELAY_SERVER,
-  CONN_PROP_STUN_RELAY_UDP_PORT,
-  CONN_PROP_STUN_RELAY_TCP_PORT,
-  CONN_PROP_STUN_RELAY_SSLTCP_PORT,
-  CONN_PROP_STUN_RELAY_USERNAME,
-  CONN_PROP_STUN_RELAY_PASSWORD,
 };
 
 enum
@@ -135,19 +104,14 @@ static void
 set_stream_playing (DBusGProxy *proxy, gboolean play, gpointer user_data);
 static void
 set_stream_sending (DBusGProxy *proxy, gboolean play, gpointer user_data);
+static void
+start_telephony_event (DBusGProxy *proxy, guchar event, gpointer user_data);
+static void
+stop_telephony_event (DBusGProxy *proxy, gpointer user_data);
 
 static void
 stop_stream (TpStreamEngineStream *self);
 
-gboolean
-g_object_has_property (GObject *object, const gchar *property)
-{
-  GObjectClass *klass;
-
-  klass = G_OBJECT_GET_CLASS (object);
-  return NULL != g_object_class_find_property (klass, property);
-}
-
 static void
 _remove_video_sink (TpStreamEngineStream *stream, GstElement *sink)
 {
@@ -167,34 +131,12 @@ _remove_video_sink (TpStreamEngineStream *stream, GstElement *sink)
   gst_bin_remove (GST_BIN (pipeline), sink);
 }
 
-#ifdef MAEMO_OSSO_SUPPORT
-static void
-media_server_proxy_cleanup (TpStreamEngineStream *self);
-#endif
-
 static void
 tp_stream_engine_stream_dispose (GObject *object)
 {
   TpStreamEngineStream *stream = TP_STREAM_ENGINE_STREAM (object);
   TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (stream);
 
-#ifdef MAEMO_OSSO_SUPPORT
-  if (priv->media_server_proxy)
-    media_server_proxy_cleanup (stream);
-#endif
-
-  if (priv->channel_path)
-    {
-      g_free (priv->channel_path);
-      priv->channel_path = NULL;
-    }
-
-  if (priv->stun_server)
-    {
-      g_free (priv->stun_server);
-      priv->stun_server = NULL;
-    }
-
   if (priv->connection_proxy)
     {
       g_object_unref (priv->connection_proxy);
@@ -227,14 +169,19 @@ tp_stream_engine_stream_dispose (GObject *object)
       dbus_g_proxy_disconnect_signal (priv->stream_handler_proxy,
           "SetStreamSending", G_CALLBACK (set_stream_sending), stream);
 
+      dbus_g_proxy_disconnect_signal (priv->stream_handler_proxy,
+          "StartTelephonyEvent", G_CALLBACK (start_telephony_event), stream);
+
+      dbus_g_proxy_disconnect_signal (priv->stream_handler_proxy,
+          "StopTelephonyEvent", G_CALLBACK (stop_telephony_event), stream);
+
       g_object_unref (priv->stream_handler_proxy);
       priv->stream_handler_proxy = NULL;
     }
 
   if (priv->fs_stream)
     {
-      if (priv->stream_started)
-        stop_stream (stream);
+      stop_stream (stream);
 
       g_signal_handler_disconnect (priv->fs_stream,
         priv->state_changed_handler_id);
@@ -255,6 +202,21 @@ tp_stream_engine_stream_dispose (GObject *object)
     G_OBJECT_CLASS (tp_stream_engine_stream_parent_class)->dispose (object);
 }
 
+static void
+tp_stream_engine_stream_finalize (GObject *object)
+{
+  TpStreamEngineStream *stream = TP_STREAM_ENGINE_STREAM (object);
+  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (stream);
+
+  if (priv->channel_path)
+    {
+      g_free (priv->channel_path);
+      priv->channel_path = NULL;
+    }
+
+  G_OBJECT_CLASS (tp_stream_engine_stream_parent_class)->finalize (object);
+}
+
 static void
 tp_stream_engine_stream_class_init (TpStreamEngineStreamClass *klass)
 {
@@ -263,6 +225,7 @@ tp_stream_engine_stream_class_init (TpStreamEngineStreamClass *klass)
   g_type_class_add_private (klass, sizeof (TpStreamEngineStreamPrivate));
 
   object_class->dispose = tp_stream_engine_stream_dispose;
+  object_class->finalize = tp_stream_engine_stream_finalize;
 
   signals[STREAM_CLOSED] =
     g_signal_new ("stream-closed",
@@ -288,24 +251,6 @@ tp_stream_engine_stream_init (TpStreamEngineStream *self)
 {
 }
 
-static void
-check_start_stream (TpStreamEngineStream *stream)
-{
-  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (stream);
-
-  DEBUG (stream, "stream_start_scheduled = %d; stream_started = %d",
-    priv->stream_start_scheduled, priv->stream_started);
-
-  if (priv->stream_start_scheduled && !priv->stream_started)
-    {
-      if (farsight_stream_get_state (priv->fs_stream) == FARSIGHT_STREAM_STATE_CONNECTED)
-        {
-          farsight_stream_start (priv->fs_stream);
-          priv->stream_started = TRUE;
-        }
-     }
-}
-
 typedef struct _method_call_ctx method_call_ctx;
 
 struct _method_call_ctx
@@ -338,30 +283,77 @@ cb_fs_state_changed (FarsightStream *stream,
 {
   TpStreamEngineStream *self = TP_STREAM_ENGINE_STREAM (user_data);
   TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
+  const gchar *state_str = "invalid!", *dir_str = "invalid!";
 
-  switch (state) {
-    case FARSIGHT_STREAM_STATE_DISCONNECTED:
-          DEBUG (self, "stream %p disconnected", stream);
-          break;
-    case FARSIGHT_STREAM_STATE_CONNECTING:
-          DEBUG (self, "stream %p connecting", stream);
-          break;
-    case FARSIGHT_STREAM_STATE_CONNECTED:
-          DEBUG (self, "stream %p connected", stream);
-          /* start the stream if its supposed to be playing already*/
-          check_start_stream (self);
-          break;
-  }
-
-  if (priv->stream_handler_proxy)
+  switch (state)
     {
-      method_call_ctx *ctx = g_slice_new0 (method_call_ctx);
+    case FARSIGHT_STREAM_STATE_DISCONNECTED:
+      state_str = "disconnected";
+      break;
+    case FARSIGHT_STREAM_STATE_CONNECTING:
+      state_str = "connecting";
+      break;
+    case FARSIGHT_STREAM_STATE_CONNECTED:
+      state_str = "connected";
+      break;
+    }
 
-      ctx->stream = self;
-      ctx->method = "Media.StreamHandler::StreamState";
+  switch (dir)
+    {
+    case FARSIGHT_STREAM_DIRECTION_NONE:
+      dir_str = "none";
+      break;
+    case FARSIGHT_STREAM_DIRECTION_SENDONLY:
+      dir_str = "send";
+      break;
+    case FARSIGHT_STREAM_DIRECTION_RECEIVEONLY:
+      dir_str = "receive";
+      break;
+    case FARSIGHT_STREAM_DIRECTION_BOTH:
+      dir_str = "both";
+      break;
+    case FARSIGHT_STREAM_DIRECTION_LAST:
+      break;
+    }
 
-      tp_media_stream_handler_stream_state_async (
-        priv->stream_handler_proxy, state, async_method_callback, ctx);
+  DEBUG (self, "stream %p, state: %s, direction: %s", stream, state_str,
+      dir_str);
+
+  if (priv->state != state || priv->dir != dir)
+    {
+      tp_stream_engine_emit_stream_state_changed (tp_stream_engine_get (),
+        priv->channel_path, self->stream_id, state, dir);
+    }
+
+  if (priv->state != state)
+    {
+      if (priv->stream_handler_proxy)
+        {
+          method_call_ctx *ctx = g_slice_new0 (method_call_ctx);
+
+          ctx->stream = self;
+          ctx->method = "Media.StreamHandler::StreamState";
+
+          tp_media_stream_handler_stream_state_async (
+            priv->stream_handler_proxy, state, async_method_callback, ctx);
+        }
+
+      priv->state = state;
+    }
+
+  if (priv->dir != dir)
+    {
+      if ((priv->dir & FARSIGHT_STREAM_DIRECTION_RECEIVEONLY) !=
+          (dir & FARSIGHT_STREAM_DIRECTION_RECEIVEONLY))
+        {
+          gboolean receiving =
+            ((dir & FARSIGHT_STREAM_DIRECTION_RECEIVEONLY) ==
+             FARSIGHT_STREAM_DIRECTION_RECEIVEONLY);
+          tp_stream_engine_emit_receiving (tp_stream_engine_get (),
+              priv->channel_path, self->stream_id, receiving);
+        }
+
+      priv->dir = dir;
     }
 }
 
@@ -721,7 +713,14 @@ set_remote_codecs (DBusGProxy *proxy, GPtrArray *codecs, gpointer user_data)
   }
   fs_codecs = g_list_reverse (fs_codecs);
 
-  farsight_stream_set_remote_codecs (priv->fs_stream, fs_codecs);
+  if (!farsight_stream_set_remote_codecs (priv->fs_stream, fs_codecs)) {
+    /*
+     * Call the error method with the proper thing here
+     */
+    g_warning("Negotiation failed");
+    tp_stream_engine_stream_error (self, 0, "Codec negotiation failed");
+    return;
+  }
 
   supp_codecs = fs_codecs_to_tp (
       farsight_stream_get_codec_intersection (priv->fs_stream));
@@ -763,34 +762,9 @@ stop_stream (TpStreamEngineStream *self)
     {
       GstElement *sink = farsight_stream_get_sink (priv->fs_stream);
       _remove_video_sink (self, sink);
-      farsight_stream_stop (priv->fs_stream);
-    }
-  else
-    {
-      farsight_stream_stop (priv->fs_stream);
     }
 
-  priv->stream_started = FALSE;
-
-#ifdef MAEMO_OSSO_SUPPORT
-  if (priv->media_server_proxy != NULL)
-    {
-      GError *error = NULL;
-
-      DEBUG (self, "re-enabling media server");
-
-      com_nokia_osso_media_server_enable (
-          DBUS_G_PROXY (priv->media_server_proxy), &error);
-      if (error)
-      {
-        g_message ("unable to re-enable media server: %s", error->message);
-        g_error_free (error);
-      }
-
-      media_server_proxy_cleanup (self);
-    }
-#endif
-
+  farsight_stream_stop (priv->fs_stream);
 }
 
 static void
@@ -805,10 +779,10 @@ set_stream_playing (DBusGProxy *proxy, gboolean play, gpointer user_data)
 
   if (play)
     {
-      priv->stream_start_scheduled = TRUE;
-      check_start_stream (self);
+      priv->playing = TRUE;
+      farsight_stream_start (priv->fs_stream);
     }
-  else if (priv->stream_started)
+  else if (priv->playing)
     {
       stop_stream (self);
     }
@@ -827,6 +801,35 @@ set_stream_sending (DBusGProxy *proxy, gboolean send, gpointer user_data)
   farsight_stream_set_sending (priv->fs_stream, send);
 }
 
+static void
+start_telephony_event (DBusGProxy *proxy, guchar event, gpointer user_data)
+{
+  TpStreamEngineStream *self = TP_STREAM_ENGINE_STREAM (user_data);
+  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
+
+  g_assert (priv->fs_stream != NULL);
+
+  DEBUG (self, "called with event %u", event);
+
+  /* this week, volume is 8, for the sake of argument... */
+  if (!farsight_stream_start_telephony_event (priv->fs_stream, event, 8))
+    DEBUG (self, "sending event %u failed", event);
+}
+
+static void
+stop_telephony_event (DBusGProxy *proxy, gpointer user_data)
+{
+  TpStreamEngineStream *self = TP_STREAM_ENGINE_STREAM (user_data);
+  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
+
+  g_assert (priv->fs_stream != NULL);
+
+  DEBUG (self, "called");
+
+  if (!farsight_stream_stop_telephony_event (priv->fs_stream))
+    DEBUG (self, "stopping event failed");
+}
+
 static void
 close (DBusGProxy *proxy, gpointer user_data)
 {
@@ -838,30 +841,77 @@ close (DBusGProxy *proxy, gpointer user_data)
   g_signal_emit (self, signals[STREAM_CLOSED], 0);
 }
 
+static void
+set_stream_properties (TpStreamEngineStream *self,
+                       TpStreamEngineStreamProperties *props)
+{
+  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
+  FarsightStream *stream = priv->fs_stream;
+  const gchar *transmitter = "rawudp";
+
+  if (props->nat_traversal == NULL ||
+      !strcmp (props->nat_traversal, "gtalk-p2p"))
+    {
+      transmitter = "libjingle";
+    }
+
+  if (g_object_has_property ((GObject *) stream, "transmitter"))
+    {
+      DEBUG (self, "setting farsight transmitter to %s", transmitter);
+      g_object_set (stream, "transmitter", transmitter, NULL);
+    }
+
+  /* transmitter should have been created as a result of setting transmitter-name */
+  if (g_object_has_property ((GObject *) stream, "transmitter-object"))
+    {
+      GObject *xmit = NULL;
+      g_object_get (stream, "transmitter-object", &xmit, NULL);
+      if (xmit != NULL)
+        {
+          if ((props->relay_token != NULL) && g_object_has_property (xmit, "relay-token"))
+            {
+              DEBUG (self, "setting farsight relay-token to %s", props->relay_token);
+              g_object_set (xmit, "relay-token", props->relay_token, NULL);
+            }
+
+          if ((props->stun_server != NULL) && g_object_has_property (xmit, "stun-ip"))
+            {
+              DEBUG (self, "setting farsight stun-ip to %s", props->stun_server);
+              g_object_set (xmit, "stun-ip", props->stun_server, NULL);
+
+              if (props->stun_port != 0)
+                {
+                  DEBUG (self, "setting farsight stun-port to %u", props->stun_port);
+                  g_object_set (xmit, "stun-port", props->stun_port, NULL);
+                }
+            }
+
+          g_object_unref (xmit);
+        }
+    }
+
+}
+
 static void
 prepare_transports (TpStreamEngineStream *self)
 {
   TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
   GPtrArray *codecs;
+  method_call_ctx *ctx;
 
-  if (priv->got_connection_properties && priv->candidate_preparation_required)
-    {
-      farsight_stream_prepare_transports (priv->fs_stream);
-      method_call_ctx *ctx;
+  farsight_stream_prepare_transports (priv->fs_stream);
 
+  codecs = fs_codecs_to_tp (
+             farsight_stream_get_local_codecs (priv->fs_stream));
 
-      codecs = fs_codecs_to_tp (
-                 farsight_stream_get_local_codecs (priv->fs_stream));
+  DEBUG (self, "calling MediaStreamHandler::Ready");
 
-      DEBUG (self, "calling MediaStreamHandler::Ready");
+  ctx = g_slice_new0 (method_call_ctx);
+  ctx->stream = self;
+  ctx->method = "Media.StreamHandler::Ready";
 
-      ctx = g_slice_new0 (method_call_ctx);
-      ctx->stream = self;
-      ctx->method = "Media.StreamHandler::Ready";
-
-      tp_media_stream_handler_ready_async (
-        priv->stream_handler_proxy, codecs, async_method_callback, ctx);
-    }
+  tp_media_stream_handler_ready_async (
+    priv->stream_handler_proxy, codecs, async_method_callback, ctx);
 }
 
 static void
@@ -881,9 +931,6 @@ cb_fs_codec_changed (FarsightStream *stream,
         NULL);
     }
 
-  tp_stream_engine_emit_receiving (tp_stream_engine_get(), priv->channel_path,
-      self->stream_id, TRUE);
-
   DEBUG (self, "codec_id=%d, stream=%p", codec_id, stream);
 
   ctx = g_slice_new0 (method_call_ctx);
@@ -972,113 +1019,6 @@ cb_fs_native_candidates_prepared (FarsightStream *stream,
     priv->stream_handler_proxy, async_method_callback, ctx);
 }
 
-static void
-set_stun (TpStreamEngineStream *self)
-{
-  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
-
-  if (priv->fs_stream)
-    {
-      if (priv->stun_server && priv->stun_port)
-       {
-         g_object_set (priv->fs_stream,
-                       "stun-ip", priv->stun_server,
-                       "stun-port", priv->stun_port,
-                       NULL);
-       }
-    }
-}
-
-static void
-cb_properties_ready (TpPropsIface *iface, gpointer user_data)
-{
-  TpStreamEngineStream *self = TP_STREAM_ENGINE_STREAM (user_data);
-  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
-  GValue server = {0,}, port = {0,};
-
-  g_value_init (&server, G_TYPE_STRING);
-  g_value_init (&port, G_TYPE_UINT);
-
-  priv->got_connection_properties = TRUE;
-
-  if (tp_props_iface_get_value (iface, CONN_PROP_STUN_SERVER, &server))
-    {
-      if (tp_props_iface_get_value (iface, CONN_PROP_STUN_PORT, &port))
-        {
-          priv->stun_server = g_value_dup_string (&server);
-          priv->stun_port = g_value_get_uint (&port);
-          set_stun (self);
-        }
-    }
-
-  /* this here in case properties_ready_cb gets called after we have
-   * recieved all the streams
-   */
-  prepare_transports (self);
-}
-
-#ifdef MAEMO_OSSO_SUPPORT
-static void
-media_server_proxy_cleanup (TpStreamEngineStream *self)
-{
-  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
-  DBusGProxy *proxy;
-
-  if (priv->media_server_proxy == NULL)
-    return;
-
-  proxy = priv->media_server_proxy;
-  priv->media_server_proxy = NULL;
-  g_object_unref (proxy);
-}
-
-static void
-media_server_proxy_destroyed (DBusGProxy *proxy, gpointer user_data)
-{
-  TpStreamEngineStream *self = TP_STREAM_ENGINE_STREAM (user_data);
-
-  DEBUG (self, "media server proxy destroyed");
-  media_server_proxy_cleanup (self);
-}
-
-static void
-media_server_proxy_init (TpStreamEngineStream *self)
-{
-  TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (self);
-  GError *me_error;
-
-  DEBUG (self, "initialising media server proxy");
-
-  priv->media_server_proxy =
-    dbus_g_proxy_new_for_name (tp_get_bus(),
-                               MEDIA_SERVER_SERVICE_NAME,
-                               MEDIA_SERVER_SERVICE_OBJECT,
-                               MEDIA_SERVER_INTERFACE_NAME);
-
-  g_signal_connect (priv->media_server_proxy, "destroy",
-                    G_CALLBACK (media_server_proxy_destroyed), self);
-
-  g_message ("disabling media server");
-
-  if (!com_nokia_osso_media_server_disable (
-        DBUS_G_PROXY (priv->media_server_proxy),
-        &me_error))
-    {
-      if (me_error)
-        {
-          g_message ("failed to disable media server: %s", me_error->message);
-          g_error_free (me_error);
-        }
-      else
-        {
-          g_message ("failed to disable media server");
-        }
-
-      media_server_proxy_cleanup (self);
-    }
-}
-#endif
-
 static GstElement *
 make_src (TpStreamEngineStream *stream, guint media_type)
 {
@@ -1144,7 +1084,9 @@ make_sink (TpStreamEngineStream *stream, guint media_type)
     }
   else
     {
-      if ((elem = getenv ("FS_VIDEO_SINK")) || (elem = getenv("FS_VIDEOSINK")))
+      if ((elem = getenv ("STREAM_VIDEO_SINK")) ||
+          (elem = getenv ("FS_VIDEO_SINK")) ||
+          (elem = getenv ("FS_VIDEOSINK")))
         {
           TpStreamEngine *engine;
           DEBUG (stream, "making video sink with pipeline \"%s\"", elem);
@@ -1180,18 +1122,14 @@ tp_stream_engine_stream_go (
   FarsightSession *fs_session,
   guint id,
   guint media_type,
-  guint direction)
+  guint direction,
+  TpStreamEngineStreamProperties *props)
 {
   TpStreamEngineStreamPrivate *priv = STREAM_PRIVATE (stream);
   TpStreamEngine *engine;
   GstElement *pipeline, *src, *sink;
   gchar *conn_timeout_str;
 
-#ifdef MAEMO_OSSO_SUPPORT
-  if (media_type == TP_MEDIA_STREAM_TYPE_AUDIO)
-    media_server_proxy_init (stream);
-#endif
-
   priv->channel_path = g_strdup (channel_path);
 
   stream->stream_id = id;
@@ -1284,6 +1222,10 @@ tp_stream_engine_stream_go (
       G_TYPE_BOOLEAN, G_TYPE_INVALID);
   dbus_g_proxy_add_signal (priv->stream_handler_proxy, "SetStreamSending",
       G_TYPE_BOOLEAN, G_TYPE_INVALID);
+  dbus_g_proxy_add_signal (priv->stream_handler_proxy, "StartTelephonyEvent",
+      G_TYPE_UCHAR, G_TYPE_INVALID);
+  dbus_g_proxy_add_signal (priv->stream_handler_proxy, "StopTelephonyEvent",
+      G_TYPE_INVALID);
   dbus_g_proxy_add_signal (priv->stream_handler_proxy, "Close",
       G_TYPE_INVALID);
 
@@ -1301,11 +1243,13 @@ tp_stream_engine_stream_go (
       G_CALLBACK (set_stream_playing), stream, NULL);
   dbus_g_proxy_connect_signal (priv->stream_handler_proxy, "SetStreamSending",
       G_CALLBACK (set_stream_sending), stream, NULL);
+  dbus_g_proxy_connect_signal (priv->stream_handler_proxy,
+      "StartTelephonyEvent", G_CALLBACK (start_telephony_event), stream, NULL);
+  dbus_g_proxy_connect_signal (priv->stream_handler_proxy,
+      "StopTelephonyEvent", G_CALLBACK (stop_telephony_event), stream, NULL);
   dbus_g_proxy_connect_signal (priv->stream_handler_proxy, "Close",
       G_CALLBACK (close), stream, NULL);
 
-  priv->candidate_preparation_required = TRUE;
-
   priv->connection_proxy = tp_conn_new (
     tp_get_bus(),
     bus_name,
@@ -1320,35 +1264,8 @@ tp_stream_engine_stream_go (
       return FALSE;
     }
 
-  /* NB: we should behave nicely if the connection doesnt have properties:
-   * sure, it's unlikely, but it's not the end of the world if it doesn't ;)
-   */
-  priv->conn_props = TELEPATHY_PROPS_IFACE (tp_conn_get_interface (
-        priv->connection_proxy, TELEPATHY_PROPS_IFACE_QUARK));
+  set_stream_properties (stream, props);
 
-  if (priv->conn_props != NULL)
-    {
-      /* surely we don't need all of these properties */
-      tp_props_iface_set_mapping (priv->conn_props,
-          "stun-server", CONN_PROP_STUN_SERVER,
-          "stun-port", CONN_PROP_STUN_PORT,
-          "stun-relay-server", CONN_PROP_STUN_RELAY_SERVER,
-          "stun-relay-udp-port", CONN_PROP_STUN_RELAY_UDP_PORT,
-          "stun-relay-tcp-port", CONN_PROP_STUN_RELAY_TCP_PORT,
-          "stun-relay-ssltcp-port", CONN_PROP_STUN_RELAY_SSLTCP_PORT,
-          "stun-relay-username", CONN_PROP_STUN_RELAY_USERNAME,
-          "stun-relay-password", CONN_PROP_STUN_RELAY_PASSWORD,
-          NULL);
-
-      g_signal_connect (priv->conn_props, "properties-ready",
-                        G_CALLBACK (cb_properties_ready), stream);
-    }
-  else
-    {
-      priv->got_connection_properties = TRUE;
-    }
-
-  set_stun (stream);
   prepare_transports (stream);
 
   return TRUE;
@@ -1495,7 +1412,7 @@ tp_stream_engine_stream_set_output_window (
       return TRUE;
     }
 
-  sink = tp_stream_engine_make_video_sink (engine);
+  sink = tp_stream_engine_make_video_sink (engine, FALSE);
 
   if (sink == NULL)
     {
