@@ -39,6 +39,13 @@ typedef struct {
   GObjectClass parent_class;
 } TpStreamEngineStreamClass;
 
+typedef struct {
+  gchar *nat_traversal;
+  gchar *stun_server;
+  guint16 stun_port;
+  gchar *relay_token;
+} TpStreamEngineStreamProperties;
+
 GType tp_stream_engine_stream_get_type (void);
 
 TpStreamEngineStream* tp_stream_engine_stream_new (void);
@@ -51,7 +58,8 @@ gboolean tp_stream_engine_stream_go (
   FarsightSession *fs_session,
   guint id,
   guint media_type,
-  guint direction);
+  guint direction,
+  TpStreamEngineStreamProperties *props);
 gboolean tp_stream_engine_stream_mute_input (
   TpStreamEngineStream *chan,
   gboolean mute_state,
@@ -78,8 +86,6 @@ void tp_stream_engine_stream_error (
   guint error,
   const gchar *message);
 
-gboolean g_object_has_property (GObject *object, const gchar *property);
-
 G_END_DECLS
 
 #endif /* __TP_STREAM_ENGINE_STREAM_H__ */
