@@ -55,6 +55,7 @@
 #include "session.h"
 #include "stream.h"
 #include "types.h"
+#include "util.h"
 #include "xerrorhandler.h"
 
 #define BUS_NAME        "org.freedesktop.Telepathy.StreamEngine"
@@ -107,6 +108,7 @@ enum
   HANDLING_CHANNEL,
   NO_MORE_CHANNELS,
   RECEIVING,
+  STREAM_STATE_CHANGED,
   SHUTDOWN_REQUESTED,
   LAST_SIGNAL
 };
@@ -135,7 +137,7 @@ struct _TpStreamEnginePrivate
   guint bad_value_handler_id;
   guint bad_window_handler_id;
 
-#ifdef MAEMO_OSSO_SUPPORT
+#ifdef USE_INFOPRINT
   DBusGProxy *infoprint_proxy;
 #endif
 };
@@ -150,6 +152,7 @@ struct _WindowPair
   guint window_id;
   volatile gboolean removing;
   gboolean created;
+  void (*post_remove) (WindowPair *wp);
 };
 
 static void
@@ -178,6 +181,7 @@ _window_pairs_add (GSList **list, TpStreamEngineStream *stream, GstElement *sink
   wp->window_id = window_id;
   wp->removing = FALSE;
   wp->created = FALSE;
+  wp->post_remove = NULL;
 
   *list = g_slist_prepend (*list, wp);
 }
@@ -270,7 +274,7 @@ _window_pairs_find_by_window_id (GSList *list, guint window_id)
 }
 
 GstElement *
-tp_stream_engine_make_video_sink (TpStreamEngine *obj)
+tp_stream_engine_make_video_sink (TpStreamEngine *obj, gboolean is_preview)
 {
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (obj);
   const gchar *videosink_name;
@@ -282,15 +286,26 @@ tp_stream_engine_make_video_sink (TpStreamEngine *obj)
 
   g_assert (priv->pipeline != NULL);
 
-  if ((videosink_name = getenv ("FS_VIDEO_SINK")) || (videosink_name = getenv("FS_VIDEOSINK")))
+  if ((videosink_name = getenv ("PREVIEW_VIDEO_SINK")) ||
+      (videosink_name = getenv ("FS_VIDEO_SINK")) ||
+      (videosink_name = getenv ("FS_VIDEOSINK")))
     {
-      g_debug ("making video sink with pipeline \"%s\"", videosink_name);
+      g_debug ("making video sink for local preview with pipeline \"%s\"", videosink_name);
       sink = gst_parse_bin_from_description (videosink_name, TRUE, NULL);
     }
   else
     {
 #ifndef MAEMO_OSSO_SUPPORT
-      sink = gst_element_factory_make ("gconfvideosink", NULL);
+      if (is_preview) {
+        /* hack to leave an xvimage free for the bigger output.
+         * Most machines only have one xvport, so this helps in the majority of
+         * cases. More intelligent widgets 
+         * */
+        sink = gst_element_factory_make ("ximagesink", NULL);
+      }
+
+      if (sink == NULL)
+        sink = gst_element_factory_make ("gconfvideosink", NULL);
 
       if (sink == NULL)
         sink = gst_element_factory_make ("autovideosink", NULL);
@@ -314,6 +329,25 @@ tp_stream_engine_make_video_sink (TpStreamEngine *obj)
           g_debug ("setting sync to FALSE");
           g_object_set (G_OBJECT (sink), "sync", FALSE, NULL);
         }
+      if (g_object_has_property (G_OBJECT (sink), "qos"))
+        {
+          g_debug ("setting qos to FALSE");
+          g_object_set (G_OBJECT (sink), "qos", FALSE, NULL);
+        }
+      if (g_object_has_property (G_OBJECT (sink), "force-aspect-ratio"))
+        {
+          g_debug ("setting force-aspect-ratio to TRUE");
+          g_object_set (G_OBJECT (sink), "force-aspect-ratio", TRUE, NULL);
+        }
+      /* Setting this will make sure we can have several preview windows using
+       * the tee without any queue elements */
+      /* Without this, elements linked to the tee just block on prerolling and
+       * wait for each other to finish */
+      if (g_object_has_property (G_OBJECT (sink), "preroll-queue-len"))
+        {
+          g_debug ("setting preroll-queue-len to 1");
+          g_object_set (G_OBJECT (sink), "preroll-queue-len", TRUE, NULL);
+        }
     }
   else
     {
@@ -335,7 +369,7 @@ tp_stream_engine_make_video_sink (TpStreamEngine *obj)
     }
 
   tmp = gst_element_factory_make ("ffmpegcolorspace", NULL);
-  if (tmp != NULL);
+  if (tmp != NULL)
     {
       g_debug ("linking ffmpegcolorspace");
       gst_bin_add (GST_BIN (bin), tmp);
@@ -379,7 +413,7 @@ _add_preview_window (TpStreamEngine *obj, guint window_id, GError **error)
   g_debug ("adding preview in window %u", window_id);
 
   tee = gst_bin_get_by_name (GST_BIN (priv->pipeline), "tee");
-  sink = tp_stream_engine_make_video_sink (obj);
+  sink = tp_stream_engine_make_video_sink (obj, TRUE);
 
   if (sink == NULL)
     goto sink_failure;
@@ -408,6 +442,8 @@ _add_preview_window (TpStreamEngine *obj, guint window_id, GError **error)
       goto link_failure;
     }
 
+  g_debug ("linked tee and sink");
+
   gst_object_unref (tee);
   g_signal_emit (obj, signals[HANDLING_CHANNEL], 0);
   return TRUE;
@@ -574,7 +610,7 @@ tp_stream_engine_class_init (TpStreamEngineClass *tp_stream_engine_class)
    /**
    * TpStreamEngine::receiving:
    *
-   * Emitted whenever a stream is received
+   * Emitted whenever a stream is receiving data
    */
   signals[RECEIVING] =
     g_signal_new ("receiving",
@@ -582,9 +618,25 @@ tp_stream_engine_class_init (TpStreamEngineClass *tp_stream_engine_class)
         G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
         0,
         NULL, NULL,
-        tp_stream_engine_marshal_VOID__STRING_INT_BOOLEAN,
+        tp_stream_engine_marshal_VOID__STRING_UINT_BOOLEAN,
         G_TYPE_NONE, 3, DBUS_TYPE_G_OBJECT_PATH, G_TYPE_UINT, G_TYPE_BOOLEAN);
 
+  /**
+   * TpStreamEngine::stream-state-changed:
+   *
+   * Emitted whenever a stream's state (connectivity, or current direction)
+   * changes
+   */
+  signals[STREAM_STATE_CHANGED] =
+    g_signal_new ("stream-state-changed",
+        G_OBJECT_CLASS_TYPE (tp_stream_engine_class),
+        G_SIGNAL_RUN_LAST | G_SIGNAL_DETAILED,
+        0,
+        NULL, NULL,
+        tp_stream_engine_marshal_VOID__STRING_UINT_UINT_UINT,
+        G_TYPE_NONE, 4, DBUS_TYPE_G_OBJECT_PATH, G_TYPE_UINT, G_TYPE_UINT,
+        G_TYPE_UINT);
+
   /**
    * TpStreamEngine::shutdown:
    *
@@ -680,7 +732,7 @@ tp_stream_engine_dispose (GObject *object)
 void
 tp_stream_engine_finalize (GObject *object)
 {
-#ifdef MAEMO_OSSO_SUPPORT
+#ifdef USE_INFOPRINT
   TpStreamEngine *self = TP_STREAM_ENGINE (object);
 
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (self);
@@ -744,83 +796,178 @@ channel_closed (TpStreamEngineChannel *chan, gpointer user_data)
   g_debug ("channel closed: %p", chan);
   g_ptr_array_remove_fast (priv->channels, chan);
   g_object_unref (chan);
+
   check_if_busy (self);
 }
 
-
 static void
-_remove_defunct_preview_sinks (TpStreamEngine *engine, gboolean clear_wp_list)
+_window_pairs_remove_cb (WindowPair *wp)
 {
-  TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (engine);
-  WindowPair *wp = NULL;
+  TpStreamEngine *self = tp_stream_engine_get ();
+  TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (self);
 
-  while ((wp = _window_pairs_find_by_removing (priv->preview_windows, TRUE)) !=
-      NULL)
-    {
-      GstElement *tee;
-
-      g_debug ("%s: removing sink for preview window ID %u", G_STRFUNC,
-          wp->window_id);
-
-      if (wp->created)
-        {
-          g_assert (wp->sink != NULL);
-          tee = gst_bin_get_by_name (GST_BIN (priv->pipeline), "tee");
-          g_assert (tee != NULL);
-          gst_element_unlink (tee, wp->sink);
-          gst_object_unref (GST_OBJECT (tee));
-          g_debug ("unlinked sink from tee, now setting state to NULL");
-
-          gst_element_set_state (wp->sink, GST_STATE_NULL);
-          g_debug ("Done setting state to NULL, now removing from bin");
-          gst_bin_remove (GST_BIN (priv->pipeline), wp->sink);
-          g_debug ("Done removing from bin, calling _window_pairs_remove"
-              "refcount %d", GST_OBJECT_REFCOUNT_VALUE (wp->sink));
-        }
-      else
-        {
-          g_debug ("No sink created yet, removing window_pair");
-          g_assert (wp->sink == NULL);
-        }
-
-      if (clear_wp_list)
-        {
-          _window_pairs_remove (&(priv->preview_windows), wp);
-        }
-      else
-        {
-          wp->sink = NULL;
-          wp->created = FALSE;
-          wp->stream = NULL;
-          wp->removing = FALSE;
-        }
-      g_debug ("Done _window_pairs_remove");
-    }
-
-    check_if_busy (engine);
+  _window_pairs_remove (&(priv->preview_windows), wp);
 }
 
 static void
-_remove_defunct_output_sinks (TpStreamEngine *engine)
+_window_pairs_empty_cb (WindowPair *wp)
 {
+  wp->sink = NULL;
+  wp->created = FALSE;
+  wp->stream = NULL;
+  wp->removing = FALSE;
+  wp->post_remove = NULL;
+}
+
+static void
+_window_pairs_readd_cb (WindowPair *wp)
+{
+  TpStreamEngine *engine = tp_stream_engine_get ();
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (engine);
-  WindowPair *wp = NULL;
 
-  while ((wp = _window_pairs_find_by_removing (priv->output_windows, TRUE)) !=
-      NULL)
+  wp->sink = NULL;
+  wp->created = FALSE;
+  wp->stream = NULL;
+  wp->removing = FALSE;
+  wp->post_remove = NULL;
+
+  if (priv->pipeline_playing)
     {
-      GError *error;
-
-      g_debug ("%s: removing sink for output window ID %u", G_STRFUNC,
-          wp->window_id);
-
-      if (!tp_stream_engine_stream_set_output_window (wp->stream, 0, &error))
-        {
-          g_debug ("%s: got error: %s", G_STRFUNC, error->message);
-          g_error_free (error);
-        }
+      GError *error = NULL;
+      _add_preview_window (engine, wp->window_id, &error);
+      if (error) {
+        g_debug ("Error creating preview window: %s", error->message);
+        g_error_free (error);
+      }
     }
 
+}
+
+static void
+unblock_cb (GstPad *pad, gboolean blocked,
+    gpointer user_data)
+{
+  g_debug ("Pad unblocked successfully after removing preview sink");
+  gst_object_unref (pad);
+}
+
+static gboolean
+_remove_defunct_preview_sink_idle_callback (gpointer user_data)
+{
+  TpStreamEngine *self = tp_stream_engine_get ();
+  TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (self);
+  gboolean retval;
+  GstStateChangeReturn ret;
+
+  WindowPair *wp = user_data;
+  g_assert (wp);
+
+  g_debug ("Removing defunct preview sink for window %u", wp->window_id);
+
+  GstPad *sink_pad = gst_element_get_pad (wp->sink, "sink");
+  g_assert (sink_pad);
+
+  GstPad *tee_src_pad = gst_pad_get_peer (sink_pad);
+  g_assert (tee_src_pad);
+
+  GstElement *sink_element = gst_pad_get_parent_element (sink_pad);
+  g_assert (sink_element);
+
+  GstElement *sink_parent = GST_ELEMENT (gst_element_get_parent (sink_element));
+  g_assert (sink_parent);
+
+  GstElement *tee = gst_bin_get_by_name (GST_BIN (priv->pipeline), "tee");
+  g_assert (tee);
+
+  GstPad *tee_sink_pad = gst_element_get_pad (tee, "sink");
+  g_assert (tee_sink_pad);
+
+  GstPad *tee_peer_src_pad = gst_pad_get_peer (tee_sink_pad);
+  g_assert (tee_peer_src_pad);
+
+
+  retval = gst_bin_remove (GST_BIN (sink_parent), sink_element);
+  g_assert (retval == TRUE);
+
+  ret = gst_element_set_state (sink_element, GST_STATE_NULL);
+  g_assert (ret != GST_STATE_CHANGE_FAILURE);
+
+  if (ret == GST_STATE_CHANGE_ASYNC) {
+    ret = gst_element_get_state (sink_element, NULL, NULL, 5*GST_SECOND);
+    g_assert (ret != GST_STATE_CHANGE_FAILURE);
+  }
+
+  gst_object_unref (tee_peer_src_pad);
+  gst_object_unref (tee_sink_pad);
+  gst_object_unref (tee);
+  gst_object_unref (sink_parent);
+  gst_object_unref (sink_element);
+  gst_element_release_request_pad (tee, tee_src_pad);
+  gst_object_unref (sink_pad);
+
+  if (wp->post_remove)
+    wp->post_remove (wp);
+
+  check_if_busy (self);
+
+  gst_pad_set_blocked_async (tee_peer_src_pad, FALSE, unblock_cb, NULL);
+
+  return FALSE;
+}
+
+static void
+_remove_defunct_preview_sink_callback (GstPad *tee_peer_src_pad, gboolean blocked,
+    gpointer user_data)
+{
+  g_debug("Pad blocked, scheduling preview sink removal");
+  g_idle_add_full (G_PRIORITY_HIGH, (GSourceFunc) _remove_defunct_preview_sink_idle_callback, user_data, NULL);
+  g_main_context_wakeup (NULL);
+}
+
+static void
+_remove_defunct_preview_sink (WindowPair *wp)
+{
+  GstElement *tee = NULL;
+  GstPad *tee_sink_pad = NULL;
+  GstPad *tee_peer_src_pad = NULL;
+  TpStreamEngine *self = tp_stream_engine_get ();
+  TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (self);
+
+  if (wp->sink == NULL)
+    return;
+
+  tee = gst_bin_get_by_name (GST_BIN (priv->pipeline), "tee");
+  g_assert (tee);
+
+  tee_sink_pad = gst_element_get_pad (tee, "sink");
+  g_assert (tee_sink_pad);
+
+  gst_object_unref (tee);
+
+  tee_peer_src_pad = gst_pad_get_peer (tee_sink_pad);
+  g_assert (tee_peer_src_pad);
+
+  gst_object_unref (tee_sink_pad);
+
+  gst_pad_set_blocked_async (tee_peer_src_pad, TRUE,
+      _remove_defunct_preview_sink_callback, wp);
+}
+
+static void
+_remove_defunct_output_sink (WindowPair *wp)
+{
+  TpStreamEngine *engine = tp_stream_engine_get ();
+  GError *error;
+
+  g_debug ("%s: removing sink for output window ID %u", G_STRFUNC,
+      wp->window_id);
+
+  if (!tp_stream_engine_stream_set_output_window (wp->stream, 0, &error))
+  {
+    g_debug ("%s: got error: %s", G_STRFUNC, error->message);
+    g_error_free (error);
+  }
+
   check_if_busy (engine);
 }
 
@@ -864,7 +1011,7 @@ bus_async_handler (GstBus *bus,
   TpStreamEngine *engine = TP_STREAM_ENGINE (data);
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (engine);
   GError *error = NULL;
-  WindowPair *wp;
+  WindowPair *wp = NULL;
   gchar *error_string;
 
   GstElement *source = GST_ELEMENT (GST_MESSAGE_SRC (message));
@@ -904,17 +1051,29 @@ bus_async_handler (GstBus *bus,
 
             if (wp != NULL)
               {
+                if (wp->removing)
+                  {
+                    g_debug ("%s: sink for %s window (id %u) has gone,"
+                        " and is already being removed",
+                        G_STRFUNC, wp->stream == NULL ? "preview" : "output",
+                        wp->window_id);
+                    break;
+                  }
+
                 g_debug ("%s: sink for %s window (id %u) has gone, removing",
                         G_STRFUNC, wp->stream == NULL ? "preview" : "output",
                         wp->window_id);
 
                 wp->removing = TRUE;
+                wp->post_remove = _window_pairs_remove_cb;
                 /*
                 xid = wp->window_id;
                 stream = wp->stream;
                 */
-                _remove_defunct_preview_sinks (engine, TRUE);
-                _remove_defunct_output_sinks (engine);
+                if (wp->stream)
+                  _remove_defunct_output_sink (wp);
+                else
+                  _remove_defunct_preview_sink (wp);
 
                 /* let's try recreating a new xvimagesink */
                 /*
@@ -947,13 +1106,27 @@ bus_async_handler (GstBus *bus,
             g_debug ("%s: destroying video pipeline", G_STRFUNC);
 
             for (i = priv->output_windows; i; i = i->next)
-              ((WindowPair *) i->data)->removing = TRUE;
+              {
+                WindowPair *wp = (WindowPair *) i->data;
+                if (wp->removing == FALSE)
+                  {
+                    wp->removing = TRUE;
+                    wp->post_remove = _window_pairs_empty_cb;
+                    _remove_defunct_output_sink (wp);
+                  }
+              }
 
             for (i = priv->preview_windows; i; i = i->next)
-              ((WindowPair *) i->data)->removing = TRUE;
+              {
+                WindowPair *wp = (WindowPair *) i->data;
+                if (wp->removing == FALSE)
+                  {
+                    wp->removing = TRUE;
+                    wp->post_remove = _window_pairs_empty_cb;
+                    _remove_defunct_preview_sink (wp);
+                  }
+              }
 
-            _remove_defunct_preview_sinks (engine, FALSE);
-            _remove_defunct_output_sinks (engine);
             gst_element_set_state (priv->pipeline, GST_STATE_NULL);
             gst_object_unref (priv->pipeline);
             priv->pipeline = NULL;
@@ -1051,6 +1224,13 @@ bus_sync_handler (GstBus *bus, GstMessage *message, gpointer data)
       g_debug ("setting sync to FALSE");
       g_object_set (G_OBJECT (GST_MESSAGE_SRC (message)), "sync", FALSE, NULL);
     }
+  if (g_object_has_property (G_OBJECT (GST_MESSAGE_SRC (message)),
+          "force-aspect-ratio"))
+    {
+      g_debug ("setting force-aspect-ratio to TRUE");
+      g_object_set (G_OBJECT (GST_MESSAGE_SRC (message)), 
+          "force-aspect-ratio", TRUE, NULL);
+    }
 
   gst_message_unref (message);
 
@@ -1111,10 +1291,13 @@ _create_pipeline (TpStreamEngine *obj)
   if (!filter)
     {
       filter = gst_caps_new_simple ("video/x-raw-yuv",
+           "framerate", GST_TYPE_FRACTION, 15, 1,
+#ifdef MAEMO_OSSO_SUPPORT
            "width", G_TYPE_INT, 176,
            "height", G_TYPE_INT, 144,
-           "framerate", GST_TYPE_FRACTION, 15, 1,
-#ifndef MAEMO_OSSO_SUPPORT
+#else
+           "width", G_TYPE_INT, 352,
+           "height", G_TYPE_INT, 288,
            "format", GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I', '4', '2', '0'),
 #endif
            NULL);
@@ -1138,7 +1321,7 @@ _create_pipeline (TpStreamEngine *obj)
     }
 
   tmp = gst_element_factory_make ("ffmpegcolorspace", NULL);
-  if (tmp != NULL);
+  if (tmp != NULL)
     {
       g_debug ("linking ffmpegcolorspace");
       gst_bin_add (GST_BIN (priv->pipeline), tmp);
@@ -1185,10 +1368,12 @@ tp_stream_engine_get_pipeline (TpStreamEngine *obj)
 
 
 static gboolean
-_remove_defunct_sinks_idle_cb (TpStreamEngine *engine)
+_remove_defunct_sinks_idle_cb (WindowPair *wp)
 {
-  _remove_defunct_preview_sinks (engine, TRUE);
-  _remove_defunct_output_sinks (engine);
+  if (wp->stream)
+    _remove_defunct_output_sink (wp);
+  else
+    _remove_defunct_preview_sink (wp);
 
   return FALSE;
 }
@@ -1211,24 +1396,30 @@ gboolean tp_stream_engine_add_preview_window (TpStreamEngine *obj,
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (obj);
   WindowPair *wp;
 
-  g_debug ("%s: called", G_STRFUNC);
+  g_debug ("%s: called for %u", G_STRFUNC, window_id);
 
   if (priv->pipeline == NULL)
     {
       _create_pipeline (obj);
     }
 
-  /* try and remove any sinks which have removing = TRUE to free up Xv ports */
-  _remove_defunct_preview_sinks (obj, TRUE);
-  _remove_defunct_output_sinks (obj);
-
   wp = _window_pairs_find_by_window_id (priv->preview_windows, window_id);
 
   if (wp != NULL)
     {
-      *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
-        "window ID %u is already a preview window", window_id);
-      return FALSE;
+      if (wp->removing && wp->post_remove == _window_pairs_remove_cb)
+        {
+          g_debug ("window ID %u is already a preview window being removed, will be re-added", window_id);
+          wp->post_remove = _window_pairs_readd_cb;
+          return TRUE;
+        }
+      else
+        {
+          g_debug ("window ID %u is already a preview window", window_id);
+          *error = g_error_new (TELEPATHY_ERRORS, InvalidArgument,
+              "window ID %u is already a preview window", window_id);
+          return FALSE;
+        }
     }
 
   if (!priv->pipeline_playing)
@@ -1281,8 +1472,9 @@ bad_window_cb (TpStreamEngineXErrorHandler *handler,
   /* set removing to TRUE so that we know this window ID is being removed and X
    * errors can be ignored */
   wp->removing = TRUE;
+  wp->post_remove = _window_pairs_remove_cb;
 
-  g_idle_add ((GSourceFunc) _remove_defunct_sinks_idle_cb, engine);
+  g_idle_add_full (G_PRIORITY_HIGH, (GSourceFunc) _remove_defunct_sinks_idle_cb, wp, NULL);
   g_main_context_wakeup (NULL);
 
   return TRUE;
@@ -1312,17 +1504,24 @@ bad_misc_cb (TpStreamEngineXErrorHandler *handler,
       return FALSE;
     }
 
-  if (!wp->removing)
+  if (wp->removing)
     {
-      g_debug ("%s: BadDrawable(%u) for a %s window not being removed, not "
-          "handling", G_STRFUNC, window_id,
-          wp->stream == NULL ? "preview" : "output");
-      return FALSE;
+      g_debug ("%s: BadDrawable(%u) for a %s window being removed, ignoring",
+          G_STRFUNC, window_id, wp->stream == NULL ? "preview" : "output");
+      return TRUE;
     }
 
-  g_debug ("%s: BadDrawable(%u) for a %s window being removed, ignoring",
+  g_debug ("%s: BadDrawable(%u) for a %s window, scheduling sink removal",
       G_STRFUNC, window_id, wp->stream == NULL ? "preview" : "output");
 
+  /* set removing to TRUE so that we know this window ID is being removed and X
+   * errors can be ignored */
+  wp->removing = TRUE;
+  wp->post_remove = _window_pairs_remove_cb;
+
+  g_idle_add_full (G_PRIORITY_HIGH, (GSourceFunc) _remove_defunct_sinks_idle_cb, wp, NULL);
+  g_main_context_wakeup (NULL);
+
   return TRUE;
 }
 
@@ -1404,6 +1603,8 @@ gboolean tp_stream_engine_remove_preview_window (TpStreamEngine *obj, guint wind
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (obj);
   WindowPair *wp;
 
+  g_debug ("%s: called for %u", G_STRFUNC, window_id);
+
   wp = _window_pairs_find_by_window_id (priv->preview_windows, window_id);
 
   if (wp == NULL)
@@ -1415,13 +1616,24 @@ gboolean tp_stream_engine_remove_preview_window (TpStreamEngine *obj, guint wind
 
   if (wp->removing)
     {
+      if (wp->post_remove == _window_pairs_readd_cb)
+        wp->post_remove = _window_pairs_remove_cb;
+
       /* already being removed, nothing to do */
       return TRUE;
     }
 
-  wp->removing = TRUE;
+  if (wp->created == FALSE)
+    {
+      g_debug ("Window not created yet, can remove right away");
+      _window_pairs_remove (&(priv->preview_windows), wp);
+      return TRUE;
+    }
 
-  _remove_defunct_preview_sinks (obj, TRUE);
+  wp->removing = TRUE;
+  wp->post_remove = _window_pairs_remove_cb;
+
+  _remove_defunct_preview_sink (wp);
 
   return TRUE;
 }
@@ -1435,9 +1647,6 @@ tp_stream_engine_add_output_window (TpStreamEngine *obj,
 {
   TpStreamEnginePrivate *priv = TP_STREAM_ENGINE_GET_PRIVATE (obj);
 
-  _remove_defunct_preview_sinks (obj, TRUE);
-  _remove_defunct_output_sinks (obj);
-
   _window_pairs_add (&(priv->output_windows), stream, sink, window_id);
 
   return TRUE;
@@ -1712,17 +1921,35 @@ tp_stream_engine_get ()
 }
 
 /*
- * tp_stream_engine_emit_received
+ * tp_stream_engine_emit_receiving
  *
  * Triggers stream engine to emit the TpStreamEngine::receiving signal
- *
  */
 void
-tp_stream_engine_emit_receiving (TpStreamEngine *obj, gchar *channel_path, guint
-    stream_id, gboolean state)
+tp_stream_engine_emit_receiving (TpStreamEngine *obj,
+                                 gchar *channel_path,
+                                 guint stream_id,
+                                 gboolean receiving)
 {
   g_signal_emit (G_OBJECT (obj), signals[RECEIVING], 0, channel_path,
-      stream_id, TRUE);
+      stream_id, receiving);
+}
+
+/*
+ * tp_stream_engine_emit_stream_state_changed
+ *
+ * Triggers stream engine to emit the TpStreamEngine::stream-state-changed
+ * signal
+ */
+void
+tp_stream_engine_emit_stream_state_changed (TpStreamEngine *obj,
+                                            gchar *channel_path,
+                                            guint stream_id,
+                                            TelepathyMediaStreamState state,
+                                            TelepathyMediaStreamDirection direction)
+{
+  g_signal_emit (G_OBJECT (obj), signals[STREAM_STATE_CHANGED], 0,
+      channel_path, stream_id, state, direction);
 }
 
 /**
